﻿Imports System.Windows.Forms

Module Startup

#Region " Declarations "
  Private Const SW_HIDE As Integer = 0
  Private Const SW_SHOWNORMAL As Integer = 1

  Private Declare Auto Function FindWindow Lib "user32.dll" (ByVal lpClassName As String, _
      ByVal lpWindowName As String) As IntPtr
  Private Declare Function ShowWindow Lib "user32.dll" (ByVal hWnd As IntPtr, _
      ByVal nCmdShow As Integer) As Boolean
#End Region

#Region " Public Methods "
  Public Sub Main(ByVal args() As String)
    Console.Title = My.Application.Info.AssemblyName

    If args Is Nothing OrElse args.Length = 0 Then
      _RunGui()
    Else
      _RunConsole(args)
    End If
  End Sub

  Friend Sub FixIntelHexFile(ByVal inFile As String, ByVal outFile As String)
    Dim file As New IO.StreamReader(inFile)
    Dim buffer As New Text.StringBuilder()

    While Not file.EndOfStream
      Dim str As String = file.ReadLine()
      Dim data() As Short = StringToShortArray(str)

      'Remove checksum byte
      Array.Resize(data, data.Length - 1)

      'Append the existing line (minus the checksum byte) and our new checksum byte
      buffer.AppendLine(ShortArrayToString(data) & CalculateChecksum(data).ToString("X2"))
    End While
    file.Close()

    'Write the contents to a file
    Dim output As New IO.StreamWriter(outFile, False)
    output.Write(buffer.ToString())
    output.Close()
  End Sub

  Friend Function ValidateIntelHexFile(ByVal fileName As String) As Boolean
    Dim ret As Boolean = True
    Dim file As New IO.StreamReader(fileName)
    Dim buffer As New Text.StringBuilder()

    While Not file.EndOfStream
      Dim str As String = file.ReadLine()
      Dim data() As Short = StringToShortArray(str)
      Dim oldChecksum As Short = data(data.Length - 1)

      'Remove checksum byte
      Array.Resize(data, data.Length - 1)

      'Append the existing line (minus the checksum byte) and our new checksum byte
      buffer.AppendLine(ShortArrayToString(data) & CalculateChecksum(data).ToString("X2"))

      If oldChecksum <> CalculateChecksum(data) Then
        'No valid for you!
        ret = False

        Exit While
      End If
    End While
    file.Close()

    Return ret
  End Function

  Friend Function CalculateChecksum(ByVal line As String) As Short
    Return CalculateChecksum(StringToShortArray(line))
  End Function

  Friend Function CalculateChecksum(ByVal line() As Short) As Short
    Dim ret As Short

    For Each s As Short In line
      ret = Convert.ToInt16((ret + s) And 255)
    Next

    ret = Convert.ToInt16((Not ret) And 255)
    ret += Convert.ToInt16(1)

    Return Convert.ToInt16(ret And 255)
  End Function

  Friend Function StringToShortArray(ByVal line As String) As Short()
    Dim ret As Short() = Nothing
    line = line.Trim().TrimStart(":"c).Trim()

    If line.Length = 0 OrElse line.Length Mod 2 <> 0 Then
      Throw New ArgumentException("Invalid checksum line!")
    Else
      Dim length As Integer = Convert.ToInt32(line.Length / 2)

      Array.Resize(ret, length)

      For i As Integer = 0 To length - 1
        ret(i) = Convert.ToInt16(line.Substring(0, 2), 16)
        line = line.Remove(0, 2)
      Next
    End If

    Return ret
  End Function

  Friend Function ShortArrayToString(ByVal data() As Short) As String
    Dim ret As String = ":"

    For Each s As Short In data
      ret &= s.ToString("X2")
    Next

    Return ret
  End Function
#End Region

#Region " Local Methods "
  Private Sub _RunGui()
    _HideConsole()
    Application.Run(New frmMain)
  End Sub

  Private Sub _RunConsole(ByVal args() As String)
    Dim newlines As String = Environment.NewLine & ControlChars.Tab

    Console.WriteLine(My.Application.Info.AssemblyName & " " & _
        My.Application.Info.Version.ToString(3) & " - " & My.Application.Info.CompanyName)
    Console.WriteLine()

    Try
      Select Case args(0).ToLower().Trim()
        Case "h", "-h", "--h", "/h", "/?"
          Console.WriteLine("Usage: " & My.Application.Info.AssemblyName & " [option] [parameters]")
          Console.WriteLine()
          Console.WriteLine("Validate Intel Hex file:" & newlines & "FixIntelHex -v [file name]")
          Console.WriteLine("Fix Intel Hex file:" & newlines & "FixIntelHex -f [input file] [output file]")
          Console.WriteLine("Calculate line checksum:" & newlines & "FixIntelHex -c [Intel Hex line without checksum]")
          Console.WriteLine("Help:" & newlines & "FixIntelHex -h")
        Case "v", "-v", "--v", "/v"
          If ValidateIntelHexFile(args(1)) Then
            Console.WriteLine(String.Format("File {0} appears to be valid.", args(1)))
          Else
            Console.WriteLine(String.Format("File {0} does NOT appear to be valid.", args(1)))
          End If
        Case "f", "-f", "--f", "/f"
          FixIntelHexFile(args(1), args(2))

          Console.WriteLine(String.Format("File {0} was written successfully.", args(2)))
        Case "c", "-c", "--c", "/c"
          Console.WriteLine("Checksum: " & CalculateChecksum(args(1)).ToString("X2"))
      End Select
    Catch ex As Exception
      Console.WriteLine()
      Console.WriteLine("ERROR: " & ex.ToString())
      Console.WriteLine()
    End Try
  End Sub

  Private Sub _HideConsole()
    Dim hWnd As IntPtr = FindWindow(Nothing, My.Application.Info.AssemblyName)

    If hWnd <> IntPtr.Zero Then
      ShowWindow(hWnd, SW_HIDE)
    End If
  End Sub

  Private Sub _ShowConsole()
    Dim hWnd As IntPtr = FindWindow(Nothing, My.Application.Info.AssemblyName)

    If hWnd <> IntPtr.Zero Then
      ShowWindow(hWnd, SW_SHOWNORMAL)
    End If
  End Sub
#End Region

End Module
